/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MC_3DShape.h"

#include <QVector>
#include <QVector3D>

MC_3DShape :: MC_3DShape()
    :m_colour(qrand()%256,qrand()%256,qrand()%256),
      m_minPoint(1000000,1000000,1000000),m_maxPoint(-1000000,-1000000,-1000000)
{

}

void MC_3DShape :: reserveVerticesMemory(int size)
{
    m_vertices.reserve(size);
    // We have as much normals than vertices
    m_normals.reserve(size);
}

void MC_3DShape :: reserveIndicesMemory(int size)
{
    m_indices.reserve(size);
}

void MC_3DShape :: addVertex(const QVector3D& vertex)
{
    m_vertices.push_back(vertex);

    // Updates shape information
    m_minPoint.setX(qMin(vertex.x(), m_minPoint.x()));
    m_minPoint.setY(qMin(vertex.y(), m_minPoint.y()));
    m_minPoint.setZ(qMin(vertex.z(), m_minPoint.z()));

    m_maxPoint.setX(qMax(vertex.x(), m_maxPoint.x()));
    m_maxPoint.setY(qMax(vertex.y(), m_maxPoint.y()));
    m_maxPoint.setZ(qMax(vertex.z(), m_maxPoint.z()));
}

void MC_3DShape :: addNormal(const QVector3D& normal)
{
    m_normals.push_back(normal);
}

void MC_3DShape :: addIndex(GLuint index)
{
    m_indices.push_back(index);
}

void MC_3DShape :: generateNormals()
{
    m_normals.resize(m_normals.capacity()); // Make the array filled and usable directly

    /**
      The technique to determine normals from vertices use the cross product
      of the edges of each shapes.
      In order to have better normals calculation, we don't keep the normal of the shape
      but we make the sum of the shape's normal for each shapes sharing the same vertex. Then, we
      are dividing the normal by the number of shape sharing it.
      Thanks to this, the normal is rounded between all the shapes sharing it, making smoother normal effect.

      In addition, we try to use a minimal amount of memory to do this.
     */

    // We have indices / 3 normals
    // Here, we keep the normal for each shape. This is not the same as the normal by vertices
    QVector <QVector3D> shapeNormals(m_indices.size()/3);

    // So we calculate the normals for each shape using cross product of the triangles' edges
    for ( int i = 0 ; i < shapeNormals.size() ; i++)
    {
        // Get the data for the shape
        unsigned int index = i * 3;
        QVector3D v1 = m_vertices[m_indices[index]];
        QVector3D v2 = m_vertices[m_indices[index+1]];
        QVector3D v3 = m_vertices[m_indices[index+2]];

        QVector3D e1 = v2 - v1;
        QVector3D e2 = v3 - v2;

        shapeNormals[i] = QVector3D::crossProduct(e1,e2);
    }

    // Array to keep the number of normals added
    QVector<unsigned int> nbNormals(m_vertices.size(),0);

    // For all indices, we add 1 to the counter for the normal
    // and add the normal to the actual normal stored
    for ( int i = 0 ; i < m_indices.size() ; i++ )
    {
        m_normals[m_indices[i]] += shapeNormals[i/3];
        nbNormals[m_indices[i]]++;
    }

    // Finally, we round up the normals
    for ( int i = 0 ; i < m_normals.size() ; i++)
    {
        m_normals[i] /= nbNormals[i];
        m_normals[i].normalize();
    }
}
