#ifndef __MC_3DSHAPE_H__
#define __MC_3DSHAPE_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QVector>
#include <QVector3D>
#include <QtOpenGL>

class MC_3DShape
{
private:

    QVector<QVector3D> m_vertices;      /*!< */
    QVector<QVector3D> m_normals;      /*!< */
    QVector<GLuint> m_indices;          /*!< */
    QColor m_colour;                    /*!< */

    // Additional shape information
    QVector3D m_minPoint;               /*!< */
    QVector3D m_maxPoint;               /*!< */

public:

    MC_3DShape();

    void reserveVerticesMemory(int size);
    void reserveIndicesMemory(int size);

    void addVertex(const QVector3D& vertex);
    void addNormal(const QVector3D& normal);
    void addIndex(GLuint index);

    void generateNormals();

    const QVector<QVector3D>& getVertices()const { return m_vertices; }
    int getVerticesDataSize()const { return m_vertices.size() * sizeof(QVector3D); }

    const QVector<QVector3D>& getNormals()const { return m_normals; }
    int getNormalsDataSize()const { return m_normals.size() * sizeof(QVector3D); }

    const QVector<GLuint>& getIndices()const { return m_indices; }
    int getIndicesDataSize()const { return m_indices.size() * sizeof(GLuint); }
    int getIndicesNumber()const { return m_indices.size(); }

    const QColor& colour()const { return m_colour; }
    void setColour(const QColor& colour) { m_colour = colour; }

    const QVector3D& minPoint()const { return m_minPoint; }
    const QVector3D& maxPoint()const { return m_maxPoint; }
};

/*! \class MC_3DShape MC_3DShape.h "3DRenderer/MC_3DShape.h"
    \brief Represents a 3D Shape

    A 3D shape is composed of three vectors (one for the vertices, one for normals, one for the indices) and one 'optional' colour.
    In addition this class is able to bring information on minimum and maximum point of the shape.
    The class base its calculation on the fact that one vertex has one normal

    To avoid huge memory allocation during some process of the shape, additional commodity functions have been added to reserve memory before opetarions. See reserveVerticesMemory() and reserveIndicesMemory().
 */

/*! \fn MC_3DShape :: MC_3DShape()
 */

/*! \fn void MC_3DShape :: reserveVerticesMemory(int size);
    \brief Reserves memory area to store the vertices
    \param size the number of vertices the shape will contains
    Will also reserve memory for normals.
 */

/*! \fn void MC_3DShape :: reserveIndicesMemory(int size);
    \brief Reserves memory area to store the indices
    \param size the number of indices the shape will use
 */

/*! \fn void MC_3DShape :: addVertex(const QVector3D& vertex);
    \brief Adds new vertex in the shape
    \param vertex
    If additional memory is needed, the internal container will do a memory allocation to accept the new data.
    Moreover, the function will update the shape information as the minimal and maximal point of the shape.
 */

/*! \fn void MC_3DShape :: addNormal(const QVector3D& normal);
    \brief Adds new normal in the shape
    \param normal
    If additional memory is needed, the internal container will do a memory allocation to accept the new data.
 */

/*! \fn void MC_3DShape :: addIndex(GLuint index);
    \brief Adds new index in the shape
    \param index
    If additional memory is needed, the internal container will do a memory allocation to accept the new data.
 */

/*! \fn void MC_3DShape :: generateNormals();
    \brief Generates normals for the shape base on vertices and indices
    The technique to determine normals from vertices use the cross product of the edges of each triangles.
    In order to have better normals calculation, we don't keep the normal of the triangle
    but we make the sum of the triangle's normal for each shapes sharing the same vertex. Then, we
    are dividing the normal by the number of triangle sharing it.
    Thanks to this, the normal is rounded between all the triangles sharing it, making smoother normal effect.

    In addition, we try to use a minimal amount of memory to do this.
 */

/*! \fn const QVector<QVector3D>& MC_3DShape :: getVertices()const
    \brief Get the vertices of the shape
    \return
 */

/*! \fn int MC_3DShape :: getVerticesDataSize()const
    \brief Get the vertices memory size in bytes
    \return
 */

/*! \fn const QVector<QVector3D>& MC_3DShape :: getNormals()const
    \brief Get the normals of the shape
    \return
  */

/*! \fn int MC_3DShape :: getNormalsDataSize()const
    \brief Get the normals memory size in bytes
    \return
 */

/*! \fn const QVector<GLuint>& MC_3DShape :: getIndices()const
    \brief Get the indices of the shape
    \return
 */

/*! \fn int MC_3DShape :: getIndicesDataSize()const
    \brief Get the indices memory size in bytes
    \return
 */

/*! \fn int MC_3DShape :: getIndicesNumber()const
    \brief Get the number of indices
    \return
 */

/*! \fn const QColor& MC_3DShape :: colour()const
    \brief Get the colour of the shape
    \return
 */

/*! \fn void MC_3DShape :: setColour(const QColor& colour)
    \brief Set the colour for the shape
    \param colour
 */

/*! \fn const QVector3D& MC_3DShape :: minPoint()const
    \brief Gets the minimal point
    \return
 */

/*! \fn const QVector3D& MC_3DShape :: maxPoint()const
    \brief Gets the maximal point
    \return
 */

#endif
