/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "MC_3DWidget.h"

#include <QtOpenGL>
#include <QDebug>

#include "3DRenderer/MC_3DRendererGL.h"
#include "3DRenderer/MC_3DRendererGL2.h"
#include "3DRenderer/3DShaders/MC_3DShader.h"
#include "3DRenderer/3DParser/trian/MC_Trian3DParser.h"

#include "Logger/Logger.h"

MC_3DWidget :: MC_3DWidget(Logger* pLogger, QWidget *parent)
    :QGLWidget(parent),m_pRenderer(NULL),m_isOpenGL2Compatible(false),m_listWidget(&m_shapeList,NULL),m_pLogger(pLogger)
{
    this->setFocusPolicy(Qt::WheelFocus);

    qDebug() << this->format();

    QGLFormat::OpenGLVersionFlags openGLVersion = this->format().openGLVersionFlags();
    if ( openGLVersion == QGLFormat::OpenGL_Version_None )
    {
        // qFatal("No OpenGL version found");
        qDebug() << "No OpenGL version found";
    }
    else if ( ( openGLVersion & QGLFormat::OpenGL_Version_2_0 ) == QGLFormat::OpenGL_Version_2_0 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_2_1 ) == QGLFormat::OpenGL_Version_2_1 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_3_0 ) == QGLFormat::OpenGL_Version_3_0 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_3_1 ) == QGLFormat::OpenGL_Version_3_1 )
    {
        m_pRenderer = new MC_3DRendererGL2();
        m_isOpenGL2Compatible = true;
        m_shapeList.setUseVBO(true);
        qDebug() << "OpenGL Renderer 2";
    }
    else if ( ( openGLVersion & QGLFormat::OpenGL_Version_1_1 ) == QGLFormat::OpenGL_Version_1_1 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_1_2 ) == QGLFormat::OpenGL_Version_1_2 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_1_3 ) == QGLFormat::OpenGL_Version_1_3 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_1_4 ) == QGLFormat::OpenGL_Version_1_4 ||
              ( openGLVersion & QGLFormat::OpenGL_Version_1_5 ) == QGLFormat::OpenGL_Version_1_5 )
    {
        m_pRenderer = new MC_3DRendererGL();
        qDebug() << "OpenGL Renderer 1";
    }
    else
    {
        qDebug() << "OpenGL version not catch: " << this->format().openGLVersionFlags();
        qDebug() << "Unsupported OpenGL Version";
    }

    m_listWidget.hide();

    m_parsersFactory.registerParser("trian",new MC_Trian3DParser());

    connect (&refreshTimer, SIGNAL(timeout()), this, SLOT(updateGL()));

    refreshTimer.start(15);
}

MC_3DWidget :: ~MC_3DWidget()
{
    delete m_pRenderer;
}

void MC_3DWidget :: initializeGL()
{
    bool bResult = true;

    if ( m_pRenderer == NULL )
    {
        // TODO: Display error
        // Quit program
    }

    bResult &= m_pRenderer->initScene();
    qglClearColor(Qt::darkBlue);

    if ( m_isOpenGL2Compatible )
    {
        bResult &= (dynamic_cast<MC_3DRendererGL2*>(m_pRenderer))->loadShaderFiles(":/data/shaders/simple.vs",":/data/shaders/simple.fs");
    }

    if ( bResult == false )
    {
        qDebug() << "OpenGL Initialisation failed";
    }
}

void MC_3DWidget :: paintGL()
{
    bool bResult = true;

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    bResult &= m_pRenderer->draw(m_shapeList);

    if ( bResult == false )
    {

    }
}

void MC_3DWidget :: resizeGL(int width, int height)
{
    m_pRenderer->resizeScene(width,height);
}

void MC_3DWidget :: mouseMoveEvent (QMouseEvent* pMouseEvent)
{
    if ( ( pMouseEvent->buttons() & Qt::LeftButton ) == Qt::LeftButton )
    {
        int dx = pMouseEvent->x() - mousePosition.x();
        int dy = pMouseEvent->y() - mousePosition.y();

        // Here can be some confusion
        // Horizontal movement turn the camera around the Y axis
        MC_3DSatelliteCamera2 camera = m_pRenderer->camera();
        if ( ( pMouseEvent->buttons() & Qt::RightButton ) == Qt::RightButton )
        {
            camera.rotateAroundZ(dx);
        }
        else
        {
            camera.rotateAroundY(dx);
        }
        camera.rotateAroundX(dy);
        m_pRenderer->setCamera(camera);

        mousePosition = pMouseEvent->pos();

        pMouseEvent->accept();
    }
}

void MC_3DWidget :: mousePressEvent (QMouseEvent* pMouseEvent)
{
    if ( pMouseEvent->button() == Qt::LeftButton )
    {
        mousePosition = pMouseEvent->pos();
        m_clickTime.start();

        pMouseEvent->accept();
    }
}

void MC_3DWidget :: mouseReleaseEvent (QMouseEvent* pMouseEvent)
{
    if ( pMouseEvent->button() == Qt::LeftButton )
    {
        if ( m_clickTime.elapsed() < 200 )
        {
           qDebug() << "User clicked";
           m_pRenderer->getShapeFrom2DPoint(pMouseEvent->pos(),m_shapeList);
        }

        pMouseEvent->accept();
    }
}

void MC_3DWidget :: wheelEvent(QWheelEvent* pWheelEvent)
{
    int numDegree = pWheelEvent->delta();

    MC_3DSatelliteCamera2 camera = m_pRenderer->camera();
    camera.zoom(numDegree);
    m_pRenderer->setCamera(camera);
}

void MC_3DWidget :: keyPressEvent(QKeyEvent* pKeyEvent)
{
    if ( pKeyEvent->key() == Qt::Key_Escape )
    {
        qApp->quit();
        return;
    }

    MC_3DSatelliteCamera2 camera = m_pRenderer->camera();

    if ( pKeyEvent->key() == Qt::Key_Left )
    {
        if ( pKeyEvent->modifiers() == Qt::ControlModifier )
        {
            camera.rotateAroundZ(-5);
        }
        else
        {
            camera.rotateAroundY(5);
        }
    }
    if ( pKeyEvent->key() == Qt::Key_Right )
    {
        if ( pKeyEvent->modifiers() == Qt::ControlModifier )
        {
            camera.rotateAroundZ(5);
        }
        else
        {
            camera.rotateAroundY(-5);
        }
    }

    if ( pKeyEvent->key() == Qt::Key_Up )
    {
        camera.rotateAroundX(-5);
    }

    if ( pKeyEvent->key() == Qt::Key_Down )
    {
        camera.rotateAroundX(5);
    }

    if ( pKeyEvent->key() == Qt::Key_PageDown )
    {
        camera.zoom(-30);
    }

    if ( pKeyEvent->key() == Qt::Key_PageUp )
    {
        camera.zoom(30);
    }

    m_pRenderer->setCamera(camera);
}

void MC_3DWidget :: closeEvent(QCloseEvent *pCloseEvent)
{
    pCloseEvent->accept();

    qApp->quit();
}

void MC_3DWidget :: clearData()
{
    m_shapeList.clear();
}

void MC_3DWidget :: load3DFile(const QString& name, const QByteArray& file)
{
    bool bResult = m_parsersFactory.parse(file,"trian",m_shapeList,name);
    if ( bResult == false )
    {
        m_pLogger->warning("Fail to load '" + name + "'");
    }
}

void MC_3DWidget :: load3DFiles()
{
    // Construct the list of extension for the filters
    QString filters = tr("3D Files (");
    QStringList availableExtensions = m_parsersFactory.getAvailableExtensions();
    foreach(QString extension, availableExtensions)
    {
        filters.append(" *." + extension);
    }
    filters.append(")");

    QStringList fileNames = QFileDialog::getOpenFileNames(this,tr("Open 3D file"),QDir::currentPath(),filters);
    if ( !fileNames.isEmpty() )
    {
        QProgressDialog progressBar(tr("Parsing 3D files..."),QString(),0,fileNames.count(),this);
        foreach (QString fileName, fileNames)
        {
            if ( !m_shapeList.contains(fileName) )
            {
                bool bResult = m_parsersFactory.parse(fileName,m_shapeList);
                if ( bResult == false )
                {
                    m_pLogger->warning("Fail to load '" + fileName + "'");
                }
            }

            progressBar.setValue(progressBar.value()+1);
        }

        // Update the camera to see all objects correctly
        QVector3D minScenePoint = m_shapeList.minPoint();
        QVector3D maxScenePoint = m_shapeList.maxPoint();
        qreal newCameraDistance = maxScenePoint.x() - minScenePoint.x();
        newCameraDistance = qMax(newCameraDistance, maxScenePoint.y() - minScenePoint.y());
        newCameraDistance = qMax(newCameraDistance, maxScenePoint.z() - minScenePoint.z());

        MC_3DSatelliteCamera2 camera = m_pRenderer->camera();
        camera.setDistance(newCameraDistance);
        m_pRenderer->setCamera(camera);
    }
}

void MC_3DWidget :: displayShapeList()
{
   m_listWidget.show();
}
