#ifndef __MC_3DWIDGET_H__
#define __MC_3DWIDGET_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QtOpenGL>
#include <QGLFormat>
#include <QTimer>
#include <QTime>

#include "3DRenderer/3DCamera/MC_3DSatelliteCamera.h"
#include "3DRenderer/3DShaders/MC_3DShader.h"
#include "3DRenderer/MC_3DShapeModelList.h"
#include "3DRenderer/3DParser/MC_3DParserFactory.h"

#include "MC_3DShapeListWidget.h"

class MC_3DRenderer;

class Logger;

class MC_3DWidget : public QGLWidget
{
    Q_OBJECT

private:

    // 3D Stuff
    MC_3DRenderer* m_pRenderer;             /*!< Renderer for 3D scene */
    bool m_isOpenGL2Compatible;             /*!< true if the renderer is OpenGL 2 (shaders) */

    // Data handling
    MC_3DShapeModelList m_shapeList;        /*!< */

    MC_3DParserFactory m_parsersFactory;    /*!< */

    MC_3DShapeListWidget m_listWidget;      /*!< */

    // Refresh management
    QTimer refreshTimer;                    /*!< Timer to control the refresh delay */

    // Mouse management
    QTime m_clickTime;                      /*!< Time to control the time between two clics */
    QPoint mousePosition;                   /*!< */

    Logger* m_pLogger;                      /*!< */

protected:

    void initializeGL();
    void paintGL();
    void resizeGL(int width, int height);

    void mouseMoveEvent (QMouseEvent* pMouseEvent);
    void mousePressEvent (QMouseEvent* pMouseEvent);
    void mouseReleaseEvent (QMouseEvent* pMouseEvent);
    void wheelEvent(QWheelEvent* pWheelEvent);
    void keyPressEvent(QKeyEvent* pKeyEvent);

public:

    explicit MC_3DWidget(Logger* pLogger, QWidget *parent = 0);
    ~MC_3DWidget();

    Q_SLOT void clearData();
    Q_SLOT void load3DFile(const QString& name, const QByteArray& file);
    Q_SLOT void load3DFiles();
    Q_SLOT void displayShapeList();

    void closeEvent(QCloseEvent *pCloseEvent);
};

/*! \class MC_3DWidget MC_3DWidget.h "MC_3DWidget.h"
    \brief Subclass of QGLWidget to render in 3D and load shapes files

    This subclass can render 3D shapes which can be read from files
    Moreover, it implements mouse related events to allow the user to rotate the camera around the scene
 */

/*! \fn explicit MC_3DWidget :: MC_3DWidget(Logger* pLogger, QWidget* parent=0)
 *  \param pLogger logger where to store messages
 *  \param parent the parent for this widget
 */

/*! \fn MC_3DWidget :: ~MC_3DWidget()
 */

/*! \fn void MC_3DWidget ::initializeGL()
 *  \brief Initialise OpenGL (called by Qt)
 */

/*! \fn void MC_3DWidget ::paintGL()
 *  \brief Displays OpenGL stuff (called by Qt)
 */

/*! \fn void MC_3DWidget ::resizeGL(int width, int height)
 *  \brief Resize the OpenGL viewport (called by Qt)
 */

/*! \fn void MC_3DWidget ::mouseMoveEvent (QMouseEvent* pMouseEvent)
 *  \brief Handles the mouse movement events to move the camera (when a clicked is pressed)
 *  \param pMouseEvent information on the mouse
 */

/*! \fn void MC_3DWidget ::mousePressEvent (QMouseEvent* pMouseEvent)
 *  \brief Handles the mouse pressure
 *  \param pMouseEvent information on the mouse
 */

/*! \fn void MC_3DWidget ::mouseReleaseEvent (QMouseEvent* pMouseEvent)
 *  \brief Handles the mouse release
 *  \param pMouseEvent information on the mouse
 */

/*! \fn void MC_3DWidget ::wheelEvent(QWheelEvent* pWheelEvent)
 *  \brief Handles the mouse wheel actions
 *  \param pWheelEvent information on the mouse wheel
 */

/*! \fn void MC_3DWidget ::keyPressEvent(QKeyEvent* pKeyEvent)
 *  \brief Handles keyboard pressure
 *  \param pKeyEvent information on the keyboard event
 */

/*! \fn void MC_3DWidget :: clearData()
 *  \brief removes all the previously loaded datas
 */

/*! \fn void MC_3DWidget :: load3DFile(const QString& name, const QByteArray& file)
 *  \brief Loads a shape from a QByteArray
 *  \param name the name of the shape
 *  \param file the data from where to read the shape data
 */

/*! \fn void MC_3DWidget :: load3DFiles()
    \brief Opens a file dialog to load 3D files
 */

/*! \fn void MC_3DWidget :: displayShapeList()
    \brief Displays the little widget showing a list of shapes
 */

/*! \fn void MC_3DWidget :: closeEvent(QCloseEvent* pCloseEvent)
 *  \brief Handles the close event from Qt to quit the application when this widget is closed
 *  \param pCloseEvent information on the event
 */

#endif // __MC_3DWIDGET_HPP__
