/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "ECGParserFactory.h"

#include <QMap>
#include <QString>
#include <QStringList>
#include <QFile>
#include <QByteArray>

#include <QDebug>

#include "ECGParser.h"

ECGParserFactory :: ~ECGParserFactory()
{
    // The deletion has to be rigourous since we can have the same MC_3DParser
    // for multiple keys in the map. So, we get the list of values and delete the uniq values
    // one by one.
    QList<ECGParser*> parsers = m_parsers.values();
    while ( parsers.size() != 0)
    {
        delete parsers[0];
        parsers.removeAll(parsers[0]);
    }
}

void ECGParserFactory :: registerParser(const QString& extension, ECGParser* pParser)
{
    if ( pParser != NULL)
    {
        QStringList extensionsList = extension.split("|", QString::SkipEmptyParts);
        foreach(QString extensionName, extensionsList)
        {
            m_parsers[extensionName] = pParser;
        }
    }
    // We do nothing if pParser is NULL
}

bool ECGParserFactory :: parse(const QString& fileName, ECGList& ecgList)
{
    // Checks if a file exist
    if ( QFile::exists(fileName) == false )
    {
        qDebug() << "File to parse '" << fileName << "' does not exist";
        return false;
    }

    // Check the extension
    int indexLastDot = fileName.lastIndexOf('.');
    if ( indexLastDot != -1 )
    {
        // here, the index points on the '.' ; We have to move one forward
        QString extensionName = fileName.right(fileName.length() - (indexLastDot+1));
        qDebug() << "Extension found: " << extensionName;

        // We check if the extension is known
        if ( m_parsers.contains(extensionName) )
        {
            return m_parsers[extensionName]->parse(fileName, ecgList);
        }
        else
        {
            qDebug() << "Unrecognised extension '" << extensionName << "'";
            return false;
        }
    }
    else
    {
        qDebug() << "Unrecognised extension for '" << fileName << "'";
        return false;
    }
}

bool ECGParserFactory :: parse(const QByteArray& data, const QString& dataType, ECGList& ecgList, const QString& ecgName)
{
    if ( m_parsers.contains(dataType) )
    {
        return m_parsers[dataType]->parse(data,ecgList,ecgName);
    }
    else
    {
        qDebug() << "Unrecognised data type '" << dataType << "'";
        return false;
    }
}

QStringList ECGParserFactory :: getAvailableExtensions()const
{
    QStringList extensions;
    for ( QMap< QString, ECGParser*>::const_iterator itParser = m_parsers.constBegin() ; itParser != m_parsers.constEnd() ; ++itParser)
    {
        extensions << itParser.key();
    }

    return extensions;
}
