/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "NetworkManager.h"

#include <cassert>

#include "MachineDataReader.h"
#include "Network/Server/TCPServer.h"
#include "Network/Server/SSLServer.h"
#include "Network/Client/NClient.h"
#include "Network/Client/TCPClient.h"
#include "Network/Client/SSLClient.h"
#include "SSLException.h"

#include "Utils/ByteArrayUtils.h"

#include "Logger/Logger.h"

const char* NetworkManager::IP_TABLE_FILENAME = "./data/machines.txt";

NetworkManager :: NetworkManager(Logger* pLogger, QObject* parent)
    :QObject(parent),m_pLogger(pLogger)
{
    m_pLogger->debug("NetworkManager started");

    MachineDataReader* pMachineDataReader = new MachineDataReader(IP_TABLE_FILENAME);
    m_machinesList = pMachineDataReader->machineData();
    delete pMachineDataReader;

    try
    {
        m_pServer = new SSLServer(CONNECTION_PORT,pLogger,parent);
        m_isSSLEnabled = true;
    }
    catch (SSLException& )
    {
        // So, let's go in TCP
        m_pServer = new TCPServer(CONNECTION_PORT,pLogger,parent);
        m_isSSLEnabled = false;
    }

    connect(m_pServer,SIGNAL(dataReceived(QByteArray)),this,SLOT(appendData(QByteArray)));

    m_pConnectionTimer = new QTimer(this);
    connect(m_pConnectionTimer,SIGNAL(timeout()),this,SLOT(connectToMachines()));

    this->connectToMachines();
}

NetworkManager :: ~NetworkManager()
{
    delete m_pConnectionTimer;
    foreach(NClient* client, m_clients)
    {
        delete client;
    }

    delete m_pServer;

    m_pLogger->debug("NetworkManager stopped");
}

void NetworkManager :: errorOnClient(NClient *pClient)
{
    assert(pClient);
    m_clients.removeOne(pClient);
    m_clientsIpAddress.removeOne(pClient->peerAddress());
    delete pClient;
}

void NetworkManager :: clientConnected(NClient *pClient)
{
    assert(pClient);
    m_clients.append(pClient);
    m_clientsIpAddress.append(pClient->peerAddress());
    emit newClientConnected(pClient);
}

void NetworkManager :: connectToMachines()
{
    const QList<MachineData>& dataList = m_machinesList;
    foreach(MachineData data, dataList)
    {
        if ( !m_clientsIpAddress.contains(data.ipAddress()) )
        {
            NClient* pClient = NULL;
            if ( m_isSSLEnabled )
            {
                pClient = new SSLClient(m_pLogger,data.machineName(),this);
            }
            else
            {
                pClient = new TCPClient(m_pLogger,data.machineName(),this);
            }

            pClient->connectSocket(data.ipAddress(),CONNECTION_PORT);
            connect(pClient,SIGNAL(errorOccured(NClient*)),this,SLOT(errorOnClient(NClient*)));
            connect(pClient,SIGNAL(connected(NClient*)),this,SLOT(clientConnected(NClient*)));
        }
    }

    m_pConnectionTimer->start(CHECK_CONNECTION_DELAY);
}

void NetworkManager :: write(MessageType type, const QByteArray& data)
{
    foreach(NClient* client, m_clients)
    {
        this->write(client,type, data);
    }
}

void NetworkManager :: write(NClient* pClient, MessageType type, const QByteArray& data)
{
    assert(pClient);

    QByteArray dataToSend;
    ByteArrayUtils::writeNumber<int>(dataToSend,type);
    dataToSend.append(data);

    pClient->write(dataToSend);
}

void NetworkManager :: appendData(QByteArray data)
{
    m_dataReceived.append(data);
}

void NetworkManager :: clearData()
{
    m_dataReceived.clear();
}

QList<QByteArray>& NetworkManager :: dataReceived()
{
    return m_dataReceived;
}

void NetworkManager :: addMachine(QString machineIP, QString machineName)
{
    if ( !machineIP.isEmpty() && !machineName.isEmpty())
    {
        foreach (MachineData machine, m_machinesList)
        {
            if ( machine.ipAddress() == machineIP )
            {
                return;
            }
        }

        m_machinesList.append(MachineData(machineIP,machineName));
        emit machineListUpdated();
    }
}

void NetworkManager :: deleteMachine(QString machineIP)
{
    if ( machineIP.isEmpty() )
    {
        return;
    }

    foreach (MachineData machine, m_machinesList)
    {
        if ( machine.ipAddress() == machineIP )
        {
            m_machinesList.removeOne(machine);
        }
    }

    foreach (NClient* pClient, m_clients)
    {
        if ( pClient->peerAddress() == machineIP )
        {
            m_clients.removeOne(pClient);
            m_clientsIpAddress.removeOne(pClient->peerAddress());
            delete pClient;
        }
    }

    emit machineListUpdated();
}
