#ifndef __NETWORKMANAGER_H__
#define __NETWORKMANAGER_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include <QObject>
#include <QList>
#include <QTimer>

#include "Types/MachineData.h"

class NServer;
class NClient;

class Logger;

class NetworkManager : public QObject
{
    Q_OBJECT

public:
    enum MessageType
    {
        MT_PatientInfo,
        MT_ChatMessage,
        MT_Connection,
        MT_Disconnection
    };

private:

    static const char* IP_TABLE_FILENAME;
    static const quint16 CONNECTION_PORT = 5555;
    static const unsigned int CHECK_CONNECTION_DELAY = 1000;

    Logger* m_pLogger;      /*!< */

    QList<MachineData> m_machinesList;

    NServer* m_pServer;         /*!< */
    QList<NClient*> m_clients;  /*!< */
    QList<QString> m_clientsIpAddress;  /*!< */
    bool m_isSSLEnabled;    /*!< */

    QList<QByteArray> m_dataReceived;   /*!< */

    QTimer* m_pConnectionTimer;         /*!< Timer to process connection to others machine */

    Q_SLOT void connectToMachines();
    Q_SLOT void errorOnClient(NClient* pClient);
    Q_SLOT void clientConnected(NClient* pClient);

public:
    NetworkManager(Logger* pLogger, QObject* parent=0);
    ~NetworkManager();

    void write(MessageType type, const QByteArray& data);
    void write(NClient* pClient, MessageType type, const QByteArray& data);
    Q_SLOT void appendData(QByteArray data);
    QList<QByteArray>& dataReceived();
    void clearData();

    const QList<MachineData>& getMachinesList()const { return m_machinesList; }

    Q_SIGNAL void newClientConnected(NClient* pClient);

    Q_SLOT void addMachine(QString machineIP, QString machineName);
    Q_SLOT void deleteMachine(QString machineIP);
    Q_SIGNAL void machineListUpdated();
};

/*! \class NetworkManager NetworkManager.h "Network/NetworkManager.h"
    \brief Handles all the network scheme for MedCenter
    The Network scheme declares that each instance of the application has a server and connects to all other machine
    Moreover, if a machine is not reachable, the connection will be retried each CHECK_CONNECTION_DELAY ms

    When a new client is connected, the signal newClientConnected is sent
    No signal informs if data are available to read, but the function dataReceived() returns these data (the list is empty if no data are available)
*/

/*! \enum NetworkManager::MessageType NetworkManager.h "Network/NetworkManager.h"
    \brief Identifier for messages type
 */

/*! \fn NetworkManager :: NetworkManager(Logger* pLogger, QObject* parent=0)
    \param pLogger
    \param parent
 */

/*! \fn NetworkManager :: ~NetworkManager()
 */

/*! \fn void NetworkManager :: connectToMachines()
    \brief Connect this application to other machines on the network (their IPs are determined by the settings file)
 */

/*! \fn void NetworkManager :: errorOnClient(NClient* pClient)
    \brief Reports an error from a client and removed it from the connected clients list
    \param pClient pointer on the client on which the error happenned
 */

/*! \fn void NetworkManager :: clientConnected(NClient* pClient)
    \brief Handles a connection successful to a client
    Add this client to the list of clients connected
    Send the newClientConnected signal
    \param pClient the pointer to the client connected
    \sa newClientConnected
 */

/*! \fn void NetworkManager :: write(MessageType type, const QByteArray& data)
    \brief Write data to all clients
    \param type
    \param data
 */

/*! \fn void NetworkManager :: write(NClient* pClient, MessageType type, const QByteArray& data)
    \brief Sends a message to the client <a>pClient</a>
    \param pClient the client who will receive the message
    \param type
    \param data
 */

/*! \fn void NetworkManager :: appendData(QByteArray data)
    \brief Adds <a>data</a> to the data received list
    \param data
 */

/*! \fn QList<QByteArray>& NetworkManager :: dataReceived()
    \brief returns the data received from the network
    \return
 */

/*! \fn void NetworkManager :: clearData()
    \brief Clears the data received
 */

/*! \fn void NetworkManager :: newClientConnected(NClient* pClient)
    \brief Signal sent when the manager is connected to a new client
    \param pClient pointer to the new client
 */

/*! \fn const QList<MachineData>& NetworkManager :: getMachinesList()const
    \brief Gets the list of machines to connect
    \return
 */

/*! \fn void NetworkManager :: addMachine(QString machineIP, QString machineName);
    \brief Add a new machine to connect to
    \param machineIP
    \param machineName
 */

/*! \fn void NetworkManager :: deleteMachine(QString machineIP);
    \brief Removes a machine
    \param machineIP
 */

/*! \fn void NetworkManager :: machineListUpdated()
    \brief Sends a signal to inform of an update of the machines list
 */

#endif
