#ifndef __RADIOMAP_H__
#define __RADIOMAP_H__

/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "Radio.h"

#include <QList>

class QString;
class QByteArray;

class RadioMap
{
private:

    QList<Radio> m_radios;                  /*!< */

    bool isExisting(const QString& name);

public:
    void clear() { m_radios.clear(); }

    bool loadRadio(const QString& fileName);
    bool loadRadio(const QString& name, const QByteArray& data);

    const QPixmap* getMasterRadio();
    QPixmap getMasterRadio(float zoomFactor);
    const QPixmap* getRadio(unsigned int x, unsigned int y);
    QPixmap getRadio(const QPoint& radioIndex, float zoomFactor);

    unsigned int width()const;
    unsigned int height()const;
};

/*! \class RadioMap RadioMap.h "RadioMap.h"
 *  \brief Container for a list of radios
 *  The radios are schematically placed on a grid. So you can access to it using 2 coordinates.
 *  The first radio loaded is a master radio, not placed on the grid. It can be seen as a map of the grid.
 */

/*! \fn bool RadioMap::isExisting(const QString& name)
 *  \brief Search if the given name is already in the RadioMap
 *  \param name the name to search in the list of radio loaded
 *  \return true if the name is already known
 */

/*! \fn void RadioMap::clear()
    \brief Clear the radios
 */

/*! \fn bool RadioMap::loadRadio(const QString& fileName)
 *  \brief Load a radio from a file
 *  \param fileName the name of the file to load
 *  \return the newly loaded radio
 *  If this fileName is already a radio's name loaded, nothing happens
 *  \sa RadioMap::load(const QString& name, const QByteArray& data)
 */

/*! \fn bool RadioMap::loadRadio(const QString& name, const QByteArray& data)
 *  \brief Load a radio from a QByteArray
 *  \param name the name of the radio
 *  \param data the data of the radio to load
 *  \return the newly loaded radio
 *  If this name is already a radio's name loaded, nothing happens
 *  \sa RadioMap::load(const QString& fileName)
 */

/*! \fn const QPixmap* RadioMap::getMasterRadio()
 *  \brief Returns the master radio (first in the list)
 *  \return Can return NULL if no radios have been loaded
 *  \return
 */

/*! \fn QPixmap RadioMap::getMasterRadio(float zoomFactor)
 *  \brief Returns the master radio (first in the list) after processing some scaling
 *  \param zoomFactor the factor of size of the image to get
 *  \return
 */

/*! \fn const QPixmap* RadioMap::getRadio(unsigned int x, unsigned int y)
 *  \brief Returns radio at coordinate x,y.
 *  \return Can return NULL if the indexes are wrong
 *  The formula used is: y * width + x + 1
 */

/*! \fn QPixmap RadioMap::getRadio(const QPoint& radioIndex, float zoomFactor)
 *  \brief Returns radio at coordinate x,y after processing some scaling
 *  \param radioIndex the index of the radio to get
 *  \param zoomFactor the factor of size of the image to get
 *  \return
 *  The formula used is: y * width + x + 1
 */

/*! \fn unsigned int RadioMap::width()const
    \brief Get the number of radio in width
    \return the width
 */

/*! \fn unsigned int RadioMap::height()const
    \brief Get the number of radio in height
    \return the height
 */

#endif
