/*
 * Copyright 2011
 * 'MedCenter' is an interactive application for hospitals.
 * (c) Copyright 2011 - Abdelhafid Hafidi  <std.abdel[at]gmail.com>
 *                    - Alexandre Laurent  <lw.demoscene[at]gmail.com>
 *
 * [GPL]
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "RadioViewerWidget.h"

#include <QString>
#include <QList>
#include <QByteArray>
#include <QFileDialog>
#include <QProgressDialog>
#include <QImageReader>
#include <QLabel>
#include <QResizeEvent>
#include <QWheelEvent>
#include <QPainter>
#include <QDebug>

#include "RadioScrollAreaWidget.h"
#include "RadioWidget.h"

#include "RadioMap.h"

#include "Logger/Logger.h"

const float ZOOM_FACTOR = 1.5f;

RadioViewerWidget :: RadioViewerWidget(Logger* pLogger, QWidget* parent)
    :QWidget(parent),m_zoom(1),m_radioSelection(-1,-1),m_pLogger(pLogger)
{
    this->setMinimumSize(512,512);
    this->setFocusPolicy(Qt::WheelFocus);

    m_pScrollArea = new RadioScrollAreaWidget(this);
    m_pRadioMap = new RadioMap();

    m_pScrollArea->setMouseTracking(true);
}

RadioViewerWidget :: ~RadioViewerWidget()
{
    delete m_pRadioMap;
}


void RadioViewerWidget :: resizeEvent(QResizeEvent* pResizeEvent)
{
    m_pScrollArea->resize(pResizeEvent->size());
}

/*
void RadioViewerWidget :: wheelEvent(QWheelEvent* pWheelEvent)
{
    if ( pWheelEvent->modifiers() == Qt::ControlModifier )
    {
        pWheelEvent->accept();
        int numDegree = pWheelEvent->delta();

        if ( numDegree > 0 )
        {
            m_zoom++;
        }
        else
        {
            m_zoom--;
        }
    }
}
*/

void RadioViewerWidget :: mouseDoubleClickEvent(QMouseEvent* pMouseEvent)
{
    const QPixmap* pActualPixmap = m_pScrollArea->getRadio();
    if ( pActualPixmap )
    {
        if ( pMouseEvent->pos().x() < pActualPixmap->width() && pMouseEvent->pos().y() < pActualPixmap->height())
        {
            // Ok, now, we can know which radio the user double clicked on
            QPoint gridPoint(qAbs(m_pScrollArea->widget()->pos().x()) + pMouseEvent->pos().x(),
                             qAbs(m_pScrollArea->widget()->pos().y()) + pMouseEvent->pos().y());
            qDebug() << "Grid point " << gridPoint;

            const QPixmap* pPixmap = m_pRadioMap->getMasterRadio();
            if ( pPixmap != NULL )
            {
                m_radioSelection.setX(gridPoint.x()/ ((pPixmap->width() / m_pRadioMap->width())*m_zoom));
                m_radioSelection.setY(gridPoint.y()/ ((pPixmap->height() / m_pRadioMap->height())*m_zoom));
                qDebug() << "Radio " << m_radioSelection;
                m_zoom = 1;
                this->update();
            }
        }
    }
}

void RadioViewerWidget :: keyPressEvent(QKeyEvent* pKeyEvent)
{
    if ( pKeyEvent->key() == Qt::Key_Plus )
    {
        m_zoom*=ZOOM_FACTOR;
        this->update();
        pKeyEvent->accept();
    }
    else if ( pKeyEvent->key() == Qt::Key_Minus )
    {
        m_zoom/=ZOOM_FACTOR;
        this->update();
        pKeyEvent->accept();
    }
}

void RadioViewerWidget :: paintEvent(QPaintEvent* pPaintEvent)
{
    (void) pPaintEvent;
    if ( m_pScrollArea->getRadio() )
    {
        QSize pixmapSize = m_pScrollArea->getRadio()->size();
        QPainter painter(this);
        QPen pen(Qt::SolidLine);
        pen.setColor(Qt::red);
        pen.setBrush(Qt::red);
        pen.setWidth(5);
        painter.setPen(pen);
        //painter.setBrush(QColor(255,0,0));

        unsigned int radioWidth = m_pRadioMap->width();
        int edgeWidth = pixmapSize.width()/radioWidth;
        for ( unsigned int i = 0 ; i <= radioWidth ; i++ )
        {
            painter.drawLine(edgeWidth*i,0,edgeWidth*i,pixmapSize.height());
        }
    }
}

void RadioViewerWidget :: update()
{
    QPixmap radio = m_pRadioMap->getRadio(m_radioSelection,m_zoom);
    m_pScrollArea->setRadio(radio);

    QPointF mousePosFactor = m_pScrollArea->radioWidget()->mouseFactorPos();
    m_pScrollArea->ensureVisible(radio.size().width()*mousePosFactor.x(),radio.size().height()*mousePosFactor.y(),this->size().width()/2,this->size().height()/2);
}

void RadioViewerWidget :: clearData()
{
    m_pRadioMap->clear();
}

void RadioViewerWidget :: loadRadioFile(const QString& name, const QByteArray& file)
{
    bool bResult = m_pRadioMap->loadRadio(name,file);
    if ( bResult == false )
    {
        m_pLogger->warning("Fail to load '" + name + "'");
    }
}

void RadioViewerWidget :: loadRadioFiles()
{
    // Construct the list of extension for the filters
    QList<QByteArray> availableExtensions = QImageReader::supportedImageFormats();
    QString filters = tr("Radio Files (");
    foreach(QByteArray extension, availableExtensions)
    {
        filters.append(" *." + QString(extension));
    }
    filters.append(")");

    QStringList fileNames = QFileDialog::getOpenFileNames(this,tr("Open radio files"),QDir::currentPath(),filters);
    fileNames.sort();   // Make sure we have the files in the order that we want.
    if ( !fileNames.isEmpty() )
    {
        QProgressDialog progressBar(tr("Loading radio files..."),QString(),0,fileNames.count(),this);
        foreach (QString fileName, fileNames)
        {
            bool bResult = m_pRadioMap->loadRadio(fileName);
            if ( bResult == false )
            {
                m_pLogger->warning("Fail to load '" + fileName + "'");
            }

            progressBar.setValue(progressBar.value()+1);
        }
    }

    m_zoom = 1;
    this->update();
}


